#!/usr/bin/perl

# Author : Brian J. Murrell (brian@interlinx.bc.ca)
# Based heavily on Shell.pm (in fact I ripped it off completely!)
#
# Contributors : 
#
# Copyright (C) 2003 Brian J. Murrell (brian@interlinx.bc.ca)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package Libconf::Glueconf::Postfix::Main_cf;
use strict;
use vars qw(@ISA);
use Libconf qw(:helpers);;
use Libconf::Glueconf;
use Libconf::Templates::Generic::KeyValue;
@ISA = qw(Libconf::Glueconf);

=head1 NAME

PERL : Libconf::Glueconf::Postfix::Main_cf
C/PYTHON/RUBY : Postfix/Main_cf

=head1 DESCRIPTION

Libconf::Glueconf::Postfix::Main_cf is a template that handles the main.cf files from postfix.

=head1 SYNOPSIS

  $postfix = new Libconf::Glueconf::Postfix::Main_cf({filename => 'main.cf' });
  $postfix->{inet_interfaces} = 'test_host';
  $postfix->write_conf();

=head1 CONSTRUCTOR

=over

=item B<new($options)>

creates the template

  $postfix = new Libconf::Glueconf::Postfix::Main_cf({
                                                       .. arguments .. 
                                                    })

the options are the same as in L<Libconf::Templates::Generic::KeyValue>, look
there to have the details.

=back

=head1 METHODS

See <Libconf::Glueconf> for general methods

=cut

sub new {
    my ($pkg, $args) = @_;

    my $libconf = new Libconf::Templates::Generic::KeyValue(add2hash_(deepcopy($args),
                                                                      {
                                                                       separator_char => '=',
                                                                       allow_space => 1,
                                                                       handle_quote => 0,
                                                                       handle_multiples_lines => 1,
                                                                      }
                                                                     )
                                                           );
    $libconf->read_conf();
    tie my %wrapper, 'Libconf::Glueconf::Postfix::Main_cf::Wrapper', $libconf;
    bless \%wrapper, $pkg;
}

sub description { 'template for the postfix main.cf configuration file' }
package Libconf::Glueconf::Postfix::Main_cf::Wrapper;
use Libconf::Glueconf::Wrapper::Hash;
our @ISA = qw(Libconf::Glueconf::Wrapper::Hash);

sub debug { Libconf::debug(@_) }

sub DELETE {
    my ($obj, $key) = @_;
    debug("key: $key");
    my @pos = $obj->{libconf}->find_atom_pos( { type => 'KEY_VALUE', key => $key });
    foreach (@pos) {
        $obj->{libconf}->delete_atom($_);
    }
}

sub FIRSTKEY {
    my ($obj) = @_;
    debug();
    my $atom = $obj->{libconf}->get_atom(0);
    $atom->{key};
}

sub EXISTS {
    my ($obj, $key) = @_;
    debug("key : $key");
    my $pos = $obj->{libconf}->find_atom_pos( { type => 'KEY_VALUE', key => $key });
    defined $pos;
}

sub NEXTKEY {
    my ($obj, $lastkey) = @_;
    debug("lastkey : $lastkey");
    my @pos = $obj->{libconf}->find_atom_pos( { type => 'KEY_VALUE', key => $lastkey });
    $pos[-1]+1 >= $obj->{libconf}->get_size() and return undef;
    $obj->{libconf}->get_atom($pos[-1]+1)->{key};
}

sub STORE {
    my ($obj, $key, $value) = @_;
    debug("key : $key - value : $value");
    ref $value eq '' or die 'trying to store anything else than a value';
    my $index;
    my @pos = $obj->{libconf}->find_atom_pos({ type => 'KEY_VALUE', key => $key });
    if (@pos == 0) {
        $index = $obj->{libconf}->append_atom({ type => 'KEY_VALUE', key => $key });
    } else {
        $index = $pos[-1];
    }
    $obj->{libconf}->edit_atom($index, { type => 'KEY_VALUE', key => $key, value => $value });
}

sub FETCH {
    my ($obj, $key) = @_;
    debug("key : $key");
    $key eq 'libconf' and return $obj->{libconf};
    my @pos = $obj->{libconf}->find_atom_pos({ type => 'KEY_VALUE', key => $key });
    @pos > 0 or return undef;
    $obj->fetch($pos[-1], 'value');
}

1;

