package Libconf::Glueconf::Wrapper;

sub debug { Libconf::debug(@_) }

sub _fetch {
    my ($obj, $atom, $key_name, @user_data) = @_;
    $atom->{$key_name};
}

sub _store {
    my ($obj, $atom, $value, $key_name, @user_data) = @_;
    $atom->{$key_name} = $value;
}

sub _delete {
    my ($obj, $atom, $value, $key_name, @user_data) = @_;
    delete $atom->{$key_name};
}

# internal function, used by template to fetch things
sub fetch {
    my ($obj, $atompos, $key_name, @user_data) = @_;
    defined $atompos or return undef;
    my $atom = $obj->{libconf}->get_atom($atompos);
    defined $atom or return undef;
    $key_name ||= 'value';
    $obj->{libconf}{_fetch_atom_pos} and $obj->{libconf}{_atom_pos} = $atompos;
    if ($obj->{libconf}{show_comments}) {
        tie my %wrapper, 'Libconf::Glueconf::Wrapper::ValueCommentWrapper', $obj, $obj->{libconf}, $atompos, $key_name, @user_data;
        return \%wrapper;
    } else {
        $obj->_fetch($atom, $key_name, @user_data);
    }
}

# used by templates to store things
sub store {
    my ($obj, $atompos, $value, $key_name, @user_data) = @_;
    $key_name ||= 'value';
    my $atom = $obj->{libconf}->get_atom($atompos);
    if ($obj->{libconf}{show_comments}) {
        tie my %wrapper, 'Libconf::Glueconf::Wrapper::ValueCommentWrapper', $obj, $obj->{libconf}, $atompos, $key_name, @user_data;
        %wrapper = %$value;
#        $wrapper->{$key_name} = $value->{$key_name};
    } else {
        $obj->_store($atom, $value, $key_name, @user_data);
    }
}

# used to set  _fetch_atom_pos if needed, for arrays and hashes.
sub resolv_atom_pos {
    my ($obj, $ref) = @_;
    $obj->{libconf}{_fetch_atom_pos} or return;
    if (UNIVERSAL::isa($ref, 'ARRAY')) {
        $ref->[0]; #we fetch the 1st atom, it'll feed _fetch_atom_pos
    } elsif (UNIVERSAL::isa($ref, 'HASH')) {
        keys %$ref; # we reset the iterator
        each %$ref; # we fetch the 1st key, it'll feed _fetch_atom_pos
        keys %$ref; # we reset the iterator
    } else { # it's probably a scalar
        $obj->{libconf}{_fetch_atom_pos} = $ref;
    }
    $ref;
}

sub check_type {
   my ($obj, $value, $type) = @_;
   if ($obj->{libconf}{show_comments}) {
       ref($value) eq 'HASH' or die "value is not of type HASH reference";
       exists $value->{_value} or die "the HASH reference does not have a _value key";
       $value = $value->{_value};
   }
   ref($value) eq ($type eq 'SCALAR' ? '' : $type) or die "value is not of type $type reference";
}

package Libconf::Glueconf::Wrapper::ValueCommentWrapper;
sub debug { Libconf::debug(@_) }

sub TIEHASH {
    my ($pkg, $object, $libconf, $atompos, $key_name, @user_data) = @_;
    debug();
    bless {
           object => $object,
           libconf => $libconf,
           atompos => $atompos,
           key_name => $key_name,
           user_data => \@user_data,
          }, $pkg;
}

sub CLEAR {
    my ($obj) = @_;
    debug();
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    my $key_name = $obj->{key_name};
    exists $atom->{$key_name} and $obj->DELETE('_value');
    exists $atom->{comments} and $obj->DELETE('_comments');
#     my $atom = $obj->{libconf}->get_atom($atompos);
#     my $key_name = $obj->{key_name};
#     ref($key_name) eq 'CODE' or delete $atom->{$key_name};
#     delete $atom->{_comments};
}

sub DELETE {
    my ($obj, $key) = @_;
    debug("key : $key");
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    $key eq '_value' and return
      $obj->{object}->_delete($atom, $obj->{key_name}, @{$obj->{user_data}});
    $key eq '_comments' and delete $atom->{comments}, return;
    delete $atom->{$key};
}

sub FIRSTKEY {
    my ($obj) = @_;
    debug();
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    my $key_name = $obj->{key_name};
    exists $atom->{$key_name} and return '_value';
    exists $atom->{comments} and return '_comments';
    undef;
}

sub EXISTS {
    my ($obj, $key) = @_;
    debug("key : $key");
    $key eq '_value' and $key = $obj->{key_name};
    $key eq '_comments' and $key = 'comments';
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    exists $atom->{$key};
}

sub NEXTKEY {
    my ($obj, $lastkey) = @_;
    debug("ValueCommentWrapper - NEXTKEY - lastkey : $lastkey");
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    my $key_name = $obj->{key_name};
    $lastkey eq '_value' && exists $atom->{comments} and return '_comments';
    undef;
}

sub STORE {
    my ($obj, $key, $value) = @_;
    debug("key : $key - value : $value");
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    $key eq '_value' and return
      $obj->{object}->_store($atom, $value, $obj->{key_name}, @{$obj->{user_data}});
    $key eq '_comments' and $atom->{comments} = $obj->{libconf}->_complexify_comments($value), return;
    $atom->{$key} = $value;
}

sub FETCH {
    my ($obj, $key) = @_;
    debug("key : $key\n");
    my $atom = $obj->{libconf}->get_atom($obj->{atompos});
    $key eq '_value' and return
      $obj->{object}->_fetch($atom, $obj->{key_name}, @{$obj->{user_data}});
    $key eq '_comments' and return $obj->{libconf}->_simplify_comments($atom->{comments});
    $atom->{$key};
}

#fetch_comments is obsolete
#     if ($obj->{libconf}->{fetch_comments}) {
#         { comments => { map { my ($a, $b) = @$_;
#                               ($a, join("\n", map { $_->{comment} } grep {$_->{inline} == $b} @{$atom->{comments}}))
#                             } ( [normal => 0], [left_inline => -1], [right_inline => 1])
#                       },
#           _value => (ref($key_name) eq 'CODE' ? $key_name->($atom) : $atom->{$key_name})
#         }
#     } else {
#         ref($key_name) eq 'CODE' ? $key_name->($atom) : $atom->{$key_name};
#     }
# }
1;
